package	jp.co.sds_corp.report; 

import java.math.BigDecimal;
import java.net.URLEncoder;

import javax.servlet.http.HttpServletResponse;

import jp.co.sds_corp.report.source.ISource;
import jp.co.sds_corp.util.common.CmnProperty;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.script.IScriptParser;

/**
* 帳票出力の基底クラスです。
*/
public abstract class AbstractReport implements IReport
{
	static CmnLogger logger = CmnLogger.getInstance();
	IConfiguration config;
	String type;
	CmnProperty prop;
	IScriptParser	scriptParser;	// スクリプトを実行するオブジェクト
	ISource	source;	// データを取得するオブジェクト
	boolean	debug;	// デバッグフラグ

	/**
	* 設定情報で初期化します。
	* @param    config  設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		this.config = config;
		this.type = config.getAttributes().get( "type", null );
		String flg = config.getAttributes().get( "debug", "" );
		if ( flg.equals( "true" )) {
			debug = true;
		}
	}

	/**
	* 帳票を出力します。
	*/
	public void print() throws Exception {
		if ( getSource() != null ) {
			getSource().execute();
			setAttribute( "bean", getSource().getBean() );
		}
		if ( getScriptParser() != null ) {
			getScriptParser().execute();
		}
	}

	/**
	* 帳票の種別を返します。
	* @return	帳票の種別
	*/
	public String getType() {
		return this.type;
	}

	/**
	* 初期情報の変換に使用するリソースを設定します。
	* @param    prop    リソース
	*/
	public void setProperty( CmnProperty prop ) {
		this.prop = prop;
	}

	/**
	* 初期情報の変換に使用するリソースを返します。
	* @return   リソース
	*/
	public CmnProperty getProperty() {
		return prop;
	}

	/**
	* スクリプトを実行するオブジェクトを設定します。
	* @param	var		設定するデータ
	*/
	public void setScriptParser( IScriptParser var ) {
		this.scriptParser = var;
	}
	/**
	* スクリプトを実行するオブジェクトを取得します。
	* @return	スクリプトを実行するオブジェクト
	*/
	public IScriptParser getScriptParser() {
		return scriptParser;
	}

	/**
	* データを取得するオブジェクトを設定します。
	* @param	var		設定するデータ
	*/
	public void setSource( ISource var ) {
		this.source = var;
	}
	/**
	* データを取得するオブジェクトを取得します。
	* @return	データを取得するオブジェクト
	*/
	public ISource getSource() {
		return source;
	}

	/**
	* スクリプトに変数を設定します。
	* @param    name    変数名
	* @param    obj     オブジェクト
	*/
	public void setAttribute( String name, Object obj ) throws Exception {
		scriptParser.setAttribute( name, obj );
	}

	/**
	* スクリプトに設定されている変数を取得します。
	* @param    name    変数名
	* @return   オブジェクト
	*/
	public Object getAttribute( String name ) throws Exception {
		return scriptParser.getAttribute( name );
	}

	/**
	* サーブレットのヘッダを設定します。
	* @param	res		HttpServletResponse
	*/
	public void setupResponse( Object res )
		throws Exception
	{
		HttpServletResponse response = (HttpServletResponse )res;
		if ( isDebug()) {
			logger.debug( this, "setupResponse" );
		}
		response.setContentType( getContentType() );
	}
	/**
	* サーブレットのヘッダを設定します。
	* @param	res		HttpServletResponse
	* @param	filename	出力するファイル名
	*/
	public void setupResponse( Object res, String filename )
		throws Exception
	{
		setupResponse( res, filename, null );
	}
	/**
	* サーブレットのヘッダを設定します。
	* @param	res		HttpServletResponse
	* @param	filename	出力するファイル名
	*/
	public void setupResponse( Object res, String filename, String encode )
		throws Exception
	{
		HttpServletResponse response = (HttpServletResponse )res;
		if ( filename.indexOf( "." ) < 0 ) {
			filename += "." + getExtension();
		}
		if ( isDebug()) {
			logger.debug( this, "setupResponse [" + filename + ":" + encode + "]" );
		}
		if ( encode == null ) {
			String enc = URLEncoder.encode( filename, "UTF-8" );
			response.setHeader( "Content-Disposition",
							"attachment;filename=" + enc );
			response.setContentType( getContentType() );
		} else {
			String enc = new String( filename.getBytes( encode ), "8859_1" );
			response.setHeader( "Content-Disposition",
							"attachment;filename=" + enc );
			response.setContentType( getContentType() + ";charset=" + encode );
		}
	}

	/**
	* デバッグフラグを設定します。
	* @param	var		設定するデータ
	*/
	public void setDebug( boolean var ) {
		this.debug = var;
	}
	/**
	* デバッグフラグを取得します。
	* @return	デバッグフラグ
	*/
	public boolean isDebug() {
		return debug;
	}

	/**
	* ログを出力します。（スクリプト内からの使用を想定）
	* @param	message	ログメッセージ
	*/
	public void log( String message ) {
		logger.debug( this, message );
	}

	public BigDecimal testNumber(Object var) {
		return (var == null) ? new BigDecimal(0) : (BigDecimal)var;
	}

	public String checkNull(Object var, String nullDef) {
		return (var == null) ? nullDef : var.toString();
	}

	public boolean isEmpty(String var) {
		return (var == null || var.length() <= 0) ? true : false;
	}

}
